<?php

   /**
    * Bookmark import for .url -> apb
    * 0.2
    *
    * Copyright (c) 2004 by Roland Eckert
    * Licensed under the GNU GPL. For full terms see the file COPYING.
    *
    * Imports one or more .url-files into Active PHP Bookmarks
	* (http://lbstone.com/apb), e.g. Internet Explorer Bookmarks aka 'Favorites'
    *
    *
    * .url file format: (c) 1998-2000 Edward L. Blake (http://www.cyanwerks.com/file-format-url.html)
    *
    */

// settings

// try to import apb.php
if (is_readable('apb.php')) {

	include_once('apb.php');

	$IMPORT_SETTINGS['apb_host']     = $GLOBALS['apb_host'];
	$IMPORT_SETTINGS['apb_database'] = $GLOBALS['apb_database'];
	$IMPORT_SETTINGS['apb_username'] = $GLOBALS['apb_username'];
	$IMPORT_SETTINGS['apb_password'] = $GLOBALS['apb_password'];

} else {

	// otherwise change these vars so that you can connect to your database.
	$IMPORT_SETTINGS['apb_host']     = "localhost";
	$IMPORT_SETTINGS['apb_database'] = "bookmarks";
	$IMPORT_SETTINGS['apb_username'] = "";
	$IMPORT_SETTINGS['apb_password'] = "";

}

// folder with the .url files, i.e. Internet Explorer Favorites
$IMPORT_SETTINGS['favorites_folder'] = '/tmp/Favorites';

// default folder name for bookmarks originally not assigned to a folder
$IMPORT_SETTINGS["unassigned_group_name"] = 'unsorted_bookmarks';

// end of settings


/* reporting/logging functions */

function msg($str_msg) {
// screen message
    echo htmlspecialchars($str_msg)."<br>\n";
}
function report_error($str_msg){
// end script because of fatal error
	msg('Error: '.$str_msg);
	exit;
}
function import_debug($str_msg) {
// verbose output
	global $IMPORT_SETTINGS;
    if ($IMPORT_SETTINGS['verbose_output']) {msg($str_msg);}
}


/* file functions */

function verify_folderspec($folderspec) {
// checks if a folder exists
	return is_dir($folderspec);
}
function read_textfile($filespec) {
// reads a file, returns the content

	$content = "";
	if (!file_exists($filespec)) {
		report_error ("This file seems not to exist: ".$filespec);
	}
	else {
		$fp = fopen($filespec, "r");
		while (!feof($fp)) {
			$content.= fread($fp, 4096);
		}
		fclose($fp);
	}
	return $content;
}


/* database functions */

function db_connect(){
// connect to the database-server, select database, return db-connection

	global $IMPORT_SETTINGS;

	$db_link = @mysql_connect($IMPORT_SETTINGS['apb_host'], $IMPORT_SETTINGS['apb_username'], $IMPORT_SETTINGS['apb_password']);
	if (!$db_link) {report_error("Unable to connect to the database.");}

	$result = @mysql_select_db($IMPORT_SETTINGS['apb_database'], $db_link);
	if (!$result) {report_error("Unable to select the database<br>".mysql_error($db_link));}

	return $db_link;
}
function db_close($db_link){
// close connection to the db

	$result = @mysql_close($db_link);
	if (!$result) {report_error("Unable to close database connection.");}
}

function db_send_sql($str_sql, $db_link){
// send a query to the db

	$result = mysql_query($str_sql, $db_link);
	if (!$result) {
        msg('Unable to execute query: '.$str_sql);
        report_error('Reason: '.mysql_error($db_link));
    }
	else {return $result;}
}

function db_execute_sql($sql) {
// connect, send query, close connection

	$db = db_connect();
	$result = db_send_sql($sql, $db);
	db_close($db);
}

function db_execute_and_return_id_sql($sql) {
// connect, send query, get last id from INSERT, close connection

	$db = db_connect();
	$result = db_send_sql($sql, $db);
	$new_id = mysql_insert_id($db);
	db_close($db);
	return $new_id;
}

function db_read_row($sql) {
// connect, execute, read a row from the db and return it as associative array, close connection

	$db = db_connect();
	$result = db_send_sql($sql, $db);
	$aryvalues = @mysql_fetch_array($result, MYSQL_ASSOC);
	db_close($db);
	return $aryvalues;
}

function db_fetch_array($rs_pointer) {
// read a row from the db and return it as associative array
	$aryvalues = mysql_fetch_array($rs_pointer, MYSQL_ASSOC);
	return $aryvalues;
}

function db_escape_string($element) {
// escape string safe for use in SQL query
	return mysql_escape_string($element);
}

function make_drop_down_from_db($table_name, $id_field, $value_field, $selected_id, $drop_down_name) {
// creates a drop-down from a table in the database, selects specified value

	$sql = "SELECT $id_field, $value_field FROM $table_name ORDER BY $value_field";

	$db = db_connect();
	$result = db_send_sql($sql, $db);

	$row_count = 0;
	$str_drop_down = "";
	while ($drop_elements = db_fetch_array($result)) {

		$row_count++;
		$str_drop_down.= "<option value=\"" . $drop_elements[$id_field] . "\"";
		if ($selected_id==$drop_elements[$id_field]) {$str_drop_down.= " selected";}
		$str_drop_down.= ">" . $drop_elements[$value_field] . "\n";

	}
	db_close($db);

	if ($row_count==0) {$str_drop_down = "No Choices present";}
	else {$str_drop_down = "<select name=\"" . $drop_down_name . "\" size=\"1\">\n" . $str_drop_down . "</select>\n";}

	return $str_drop_down;
}


/* string functions */

function validate_only_numeric($elem) {
// filters everything bar numericals
	return (preg_replace("/[^0-9]/", "", $elem));
}


/* bookmark specific functions */

function bookmarks_reset() {
// reset bookmarks - keeps users. NOT called by default

    global $IMPORT_SETTINGS;

    if (($IMPORT_SETTINGS["reset_bookmarks"]) && (!$IMPORT_SETTINGS['dry_run'])) {
        import_debug('Resetting Active PHP Bookmarks');
        $sql = 'DELETE FROM apb_bookmarks;';
        db_execute_sql($sql);
        $sql = 'DELETE FROM apb_groups;';
        db_execute_sql($sql);
        $sql = 'DELETE FROM apb_hits;';
        db_execute_sql($sql);
    }
}

function get_unassigned_group_id() {

	global $IMPORT_SETTINGS;

	$sql = "SELECT group_id FROM apb_groups WHERE group_title='".$IMPORT_SETTINGS['unassigned_group_name']."'";
	$ary_result = db_read_row($sql);

	// does not exist? create and return id
	if (!$ary_result) {

    	$sql = "INSERT INTO apb_groups (group_parent_id,group_title,user_id,group_creation_date) VALUES".
			    " (0,'".$IMPORT_SETTINGS['unassigned_group_name']."',".$IMPORT_SETTINGS['user_id'].",now());";

        if (!$IMPORT_SETTINGS["dry_run"]) {
            $group_id = db_execute_and_return_id_sql($sql);
            import_debug("Unassigned group created: ".$IMPORT_SETTINGS["unassigned_group_name"]);
        } else {
            $group_id=0;
        }

	// does exist?
	} else {
		$group_id = $ary_result["group_id"];
        import_debug("Unassigned group found: ".$group_id);
	}

    $IMPORT_SETTINGS['unassigned_group_id'] = $group_id;
}

function get_group_id($folder) {
// insert folder to database recursively

	global $IMPORT_SETTINGS, $ary_duplicate_groups;

	// initial level zero
	$group_level = 0;

	// initial parent id zero
	$parent_id = 0;

    // zap root favorites folder part - anyone a better idea? :-)
    $folder = substr($folder, $IMPORT_SETTINGS['favorites_folder_len']);

	// no group? assign to designated folder
	if (!$folder) { import_debug('Group: '.$IMPORT_SETTINGS['unassigned_group_name']); return $IMPORT_SETTINGS['unassigned_group_id']; }

	// split folder on slash
	$ary_folder = explode("/", $folder);

	// read group_ids, if not found, create
	for ($i=0; $i<count($ary_folder); $i++) {

		// safety check - no folder name?`
		if (!$ary_folder[$i]) {continue;}

		$group_title = db_escape_string($ary_folder[$i]);

		// new parent id after first round
		if ($group_level>0) {$parent_id = $group_id;}

		$sql = "SELECT group_id FROM apb_groups WHERE group_title='$group_title' AND group_parent_id=$parent_id;";
		$ary_result = db_read_row($sql);

		// does not exist? create new group and return id
		if (!$ary_result) {

			// check if group exists previously with differrent parent id - shortcoming of apb, UNIQUE group_title (wtf?)
			$sql = "SELECT COUNT(*) AS duplicates FROM apb_groups WHERE group_title='$group_title' AND group_parent_id<>$parent_id;";
			$ary_check = db_read_row($sql);
			if ($ary_check['duplicates']>0) {
				// duplicate group found - show warning and skip
				if (empty($ary_duplicate_groups[$folder])) {$ary_duplicate_groups[$folder]=true;}
				msg('GROUP SKIPPED: '.$group_title);
				return -1;
			}

			$sql = "INSERT INTO apb_groups (group_parent_id,group_title,user_id,group_creation_date) VALUES".
				    " ($parent_id,'$group_title',".$IMPORT_SETTINGS['user_id'].",now());";

            if (!$IMPORT_SETTINGS['dry_run']) {
                $group_id = db_execute_and_return_id_sql($sql);
                import_debug('Group created: ('.$group_id.') '.$group_title);
            } else {
                $group_id=0;
            }

		// does exist?
		} else {
			$group_id = $ary_result["group_id"];
			import_debug('Group found:  ('.$group_id.') '.$group_title);
		}

		// increase group level
		$group_level++;
        flush();
	}

	// return group id
	return $group_id;

}

function get_url($filespec) {
// reads the .url file, returns URL value

	$content = read_textfile($filespec);
	preg_match("/^URL=(.+)/mi", $content, $url_array);
	if (count($url_array)==2){$url = $url_array[1];}
	else {$url="";}
	return $url;
}

function check_link_in_db($url) {
// check if link exists in db
	$sql = "SELECT bookmark_id FROM apb_bookmarks WHERE bookmark_url='".db_escape_string($url)."';";
	$ary_result = db_read_row($sql);
	return ( ($ary_result) ? true : false );
}

function enter_link_in_db($group_id, $title, $url) {
// enters the bookmark into the database if not in there

	global $IMPORT_SETTINGS;

	// create link
	$sql =  "INSERT INTO apb_bookmarks (group_id,bookmark_title,bookmark_url,bookmark_creation_date,bookmark_private,user_id) VALUES".
			" ($group_id,'".db_escape_string($title)."','".db_escape_string($url)."',now(),".$IMPORT_SETTINGS['privacy'].",".$IMPORT_SETTINGS['user_id'].")";

    if (!$IMPORT_SETTINGS["dry_run"]) {
        db_execute_sql($sql);
    }
}

function process_folder($folder) {
// process all entries in folder

    global $IMPORT_SETTINGS, $links_total, $duplicates_total, $ary_duplicate_links;

	// open folder
	$handle = opendir($folder);

	// loop until no entries left
    while ($entry = readdir($handle)) {

        // is dot file?
        if (($entry==".") || ($entry=="..")) {continue;}
		import_debug(' ');
		import_debug('Processing folder: '.$folder);

		// is folder? recursive call
        if (is_dir("$folder/$entry")) {
            process_folder("$folder/$entry");

		// is file?
        } else {

            // has .url file extension?
            if (!preg_match("/\.url$/i", $entry)) {continue;}

			import_debug('Processing file: '.$entry);

            // take out extension
            $title = str_replace(".url", "", $entry);

            // now the interesting part - return group id of this folder based on folder name
            $group_id = get_group_id($folder);
			if (!$group_id) {$duplicates_total++; continue;}

            // read .url file, parse for url
            $url = get_url("$folder/$entry");

            // no url found?
            if (!$url) {continue;}

            // don't include if javascript link
            if (($IMPORT_SETTINGS["suppress_js"]) && (preg_match("/^javascript/i", $url))) {
                import_debug('Skipped Javascript link'); continue;
            }

            // don't include if local file link
            if (($IMPORT_SETTINGS["suppress_file"]) && (preg_match("/^file/i", $url))) {
                import_debug('Skipped file link'); continue;
            }

			// check for duplicate
			if ($IMPORT_SETTINGS["suppress_duplicates"]) {
				if (check_link_in_db($url)) {

					// add to duplicates list
					if (empty($ary_duplicate_links[$url])) {$ary_duplicate_links[$url]=true;}
					$duplicates_total++;
					import_debug('Duplicate URL skipped: '.$url);
					continue;
				}
			}

            // enter link into database
            enter_link_in_db($group_id, $title, $url);
			$links_total++;
			import_debug('Link inserted: Group '.$group_id.', "'.$title.'" ('.$url.')');

			flush();

		}

	}
}

function init() {
// initialise variables

	global $IMPORT_SETTINGS, $links_total, $duplicates_total, $groups_total;

    $links_total = 0;
    $duplicates_total = 0;
    $groups_total = 0;

	$IMPORT_SETTINGS["favorites_folder"] = $_POST["favorites_folder"];
	$IMPORT_SETTINGS["user_id"] = validate_only_numeric($_POST["user_id"]);
	$IMPORT_SETTINGS["unassigned_group_name"] = $_POST["unassigned_group_name"];

	$IMPORT_SETTINGS["privacy"] = (!empty($_POST["privacy"])) ? '1' : '0';
	$IMPORT_SETTINGS["suppress_duplicates"] = (!empty($_POST["suppress_duplicates"])) ? true : false;
	$IMPORT_SETTINGS["suppress_js"] = (!empty($_POST["suppress_js"])) ? true : false;
	$IMPORT_SETTINGS["suppress_file"] = (!empty($_POST["suppress_file"])) ? true : false;
	$IMPORT_SETTINGS["reset_bookmarks"] = (!empty($_POST["reset_bookmarks"])) ? true : false;
	$IMPORT_SETTINGS["verbose_output"] = (!empty($_POST["verbose_output"])) ? true : false;
	$IMPORT_SETTINGS["dry_run"] = (!empty($_POST["dry_run"])) ? true : false;

    $IMPORT_SETTINGS['favorites_folder_len'] = strlen($IMPORT_SETTINGS['favorites_folder']) + 1;
}

function verify() {
// verify used configuration

	global $IMPORT_SETTINGS;

    import_debug('Using favorites folder: '.$IMPORT_SETTINGS['favorites_folder']);
    import_debug('User Id: '.$IMPORT_SETTINGS['user_id']);
    import_debug('Unassigned group name: '.$IMPORT_SETTINGS['unassigned_group_name']);
    import_debug('Privacy: '.$IMPORT_SETTINGS['privacy']);
    import_debug('Suppress duplicates: '.$IMPORT_SETTINGS['suppress_duplicates']);
    import_debug('Suppress Javascript links: '.$IMPORT_SETTINGS['suppress_js']);
    import_debug('Suppress file links: '.$IMPORT_SETTINGS['suppress_file']);
    import_debug('Reset Active PHP Bookmarks: '.$IMPORT_SETTINGS["reset_bookmarks"]);
    import_debug('Verbose output: '.$IMPORT_SETTINGS['verbose_output']);
    import_debug('Dry run: '.$IMPORT_SETTINGS['dry_run']);
    import_debug(' ');

    // verify favorites folder location
    if (!verify_folderspec($IMPORT_SETTINGS['favorites_folder'])) {
        report_error('Favorites folder can not be found.');}

    // verify user id
    $sql = "SELECT user_id FROM apb_users WHERE user_id='".$IMPORT_SETTINGS['user_id']."';";
    $ary_result = db_read_row($sql);
    if (!$ary_result) {report_error('unknown user.');}

    // verify unassigned bookmarks folder
    if (!$IMPORT_SETTINGS['unassigned_group_name']) {report_error('unassigned bookmarks folder name not set.');}

}

function import() {

	global $IMPORT_SETTINGS, $links_total, $duplicates_total, $groups_total, $ary_duplicate_links, $ary_duplicate_groups;

    // start import
    msg("Import started...");

    // reset bookmarks
    bookmarks_reset();

	// return/create unassigned group for bookmarks in main folder
	get_unassigned_group_id();

	// process specified folder
    process_folder($IMPORT_SETTINGS['favorites_folder']);

    // end of script
	msg(' ');
    msg("Import finished. $links_total links imported.");
	msg(' ');

	if ($ary_duplicate_links) {
		msg('There were '.$duplicates_total.' duplicate urls which have been skipped:');
		foreach ($ary_duplicate_links as $skipped=>$value) {
			msg($skipped);
		}
		msg(' ');
	}

	if ($ary_duplicate_groups) {
		msg('There were duplicate folders (APB does not allow two groups with the same name, even in separate subgroups)');
		msg('Rename these and re-run the import:');
		foreach ($ary_duplicate_groups as $skipped=>$value) {
			msg ('/'.$skipped);
		}
	}
}

function show_options() {

   	global $IMPORT_SETTINGS;

	//user name drop down
	$user_drop = make_drop_down_from_db("apb_users" , "user_id", "username", "", "user_id");

	echo "
        <html><head><title>Import Internet Explorer Favorites into Active PHP Bookmarks</title></head><body>
		<div align=\"center\">
		<h2>Import Internet Explorer Favorites into Active PHP Bookmarks</h2>

		<form action=\"ie_import.php\" method=\"POST\">
		<input type=\"hidden\" name=\"action\" value=\"import\">
		<table border=\"1\" cellspacing=\"5\" cellpadding=\"8\">

		<tr>
		<td>Modify the path to your Favorites folder
		<br><input type=\"text\" size=\"60\" name=\"favorites_folder\" value=\"".$IMPORT_SETTINGS['favorites_folder']."\"></td>
		</tr>

		<tr>
		<td>Select APB username $user_drop</td>
		</tr>

		<tr>
		<td>Unassigned group name <input type=\"text\" name=\"unassigned_group_name\" value=\"".$IMPORT_SETTINGS['unassigned_group_name']."\" size=\"30\" maxlength=\"100\"></td>
		</tr>

		<tr>
		<td><input type=\"checkbox\" name=\"privacy\"> Mark as private</td>
		</tr>

		<tr>
		<td><input type=\"checkbox\" name=\"suppress_duplicates\" checked> Suppress duplicate entries</td>
		</tr>

		<tr>
		<td><input type=\"checkbox\" name=\"suppress_js\" checked> Suppress Javascript bookmarks ('javascript:')</td>
		</tr>

		<tr>
		<td><input type=\"checkbox\" name=\"suppress_file\" checked> Suppress local file bookmarks ('file://')</td>
		</tr>

		<tr>
		<td><input type=\"checkbox\" name=\"reset_bookmarks\"> Clear Active PHP Bookmarks first. <b>BE CAREFUL. BACKUP YOUR DATA. NO UNDO.</b></td>
		</tr>

		<tr>
		<td><input type=\"checkbox\" name=\"verbose_output\" checked> Verbose output</td>
		</tr>

		<tr>
		<td><input type=\"checkbox\" name=\"dry_run\" checked> Dry Run. No changes made to database</td>
		</tr>

		<tr>
		<td align=\"center\"><input type=\"submit\" name=\"submit\" value=\"Import Bookmarks\"></td>
		</tr>
		</table>
		</form>
		</div>
        </body></html>
		 ";
}

/* main functions */

if (empty($action)) {

	show_options();

} else {

	// initialize
	init();

	// verify config
    verify();

    // start import
	import();
}

?>
